<?php
/*
Plugin Name: jQuery Pin It Button For Images
Plugin URI: http://wordpress.org/extend/plugins/jquery-pin-it-button-for-images/
Description: Highlights images on hover and adds a "Pin It" button over them for easy pinning.
Author: Marcin Skrzypiec
Version: 1.00
Author URI: http://profiles.wordpress.org/mrsztuczkens
*/

if ( ! function_exists( 'add_action' ) ) {
	echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
	exit;
}

/*
 *
 * CONSTANTS
 *
 */
define( "JPIBFI_VERSION", "1.00" );
define( "JPIBFI_METADATA", "jpibfi_meta" );
define( "JPIBFI_SELECTION_OPTIONS", "jpibfi_selection_options" );
define( "JPIBFI_VISUAL_OPTIONS", "jpibfi_visual_options" );
define( "JPIBFI_VERSION_OPTION", "jpibfi_version");

//DEFAULT PIN BUTTON IMAGE
define( "JPIBFI_IMAGE_URL", plugins_url( '/images/pinit-button.png', __FILE__ ) );
define( "JPIBFI_IMAGE_WIDTH", 65 );
define( "JPIBFI_IMAGE_HEIGHT", 41 );

/*
 *
 * FRONT END STUFF
 *
 */

/*
 * Checks if the plugin wasn't deactivated in the given post/page
 */
function jpibfi_add_plugin_to_post( $post_id ) {
	$post_meta = get_post_meta( $post_id, JPIBFI_METADATA, true );
	return empty( $post_meta )
			|| false == array_key_exists( 'jpibfi_disable_for_post', $post_meta )
			|| '1' != $post_meta['jpibfi_disable_for_post'];
}

/*
 * function copied from https://gist.github.com/wesbos/1189639
 */
function jpibfi_is_blog_page() {
	global $post;

	$post_type = get_post_type( $post );

	return ( ( is_home() || is_archive() || is_single() ) && ( $post_type == 'post' )	);
}

/*
 * True if plugin should be added to the current post/page
 */
function jpibfi_add_plugin() {

	global $post;
	$options = get_option( JPIBFI_SELECTION_OPTIONS );

	if ( is_front_page() )
		return isset( $options['show_on_home'] ) && $options['show_on_home'] == "1";
	else if ( is_single() )
		return isset( $options['show_on_single'] ) && $options['show_on_single'] == "1" ? jpibfi_add_plugin_to_post( $post->ID ) : false;
	else if ( is_page() )
		return isset( $options['show_on_page'] ) && $options['show_on_page'] == "1" ? jpibfi_add_plugin_to_post( $post->ID ) : false;
	else if ( is_category() || is_archive() || is_search() )
		return isset( $options['show_on_category'] ) && $options['show_on_category'] == "1";
	else if ( jpibfi_is_blog_page() )
		return isset( $options['show_on_blog'] ) && $options['show_on_blog'] == "1";
	return true;
}

//Adds all necessary scripts
function jpibfi_add_plugin_scripts() {
	if ( ! ( jpibfi_add_plugin() ) )
		return;

	wp_register_style( 'jquery-pin-it-button-style', plugins_url( '/css/style.css', __FILE__ ), array(), JPIBFI_VERSION, 'all' );
	wp_enqueue_style( 'jquery-pin-it-button-style' );

	wp_enqueue_script( 'jquery-pin-it-button-script', plugins_url( '/js/script.min.js', __FILE__ ), array( 'jquery' ), JPIBFI_VERSION, false );

	$visual_options = get_option( JPIBFI_VISUAL_OPTIONS );
	$selection_options = get_option( JPIBFI_SELECTION_OPTIONS );

	$parameters_array = array(
		'image_selector' 		=> $selection_options['image_selector'],
		'disabled_classes' 	=> $selection_options['disabled_classes'],
		'enabled_classes' 	=> $selection_options['enabled_classes'],
		'description_option'=> $visual_options['description_option'],
		'use_post_url' 			=> isset ( $visual_options['use_post_url'] ) ? $visual_options['use_post_url'] : '0',
		'min_image_height'	=> $selection_options['min_image_height'],
		'min_image_width'		=> $selection_options['min_image_width'],
		'site_title'				=> get_bloginfo( 'name', 'display' )
	);
	wp_localize_script( 'jquery-pin-it-button-script', 'jpibfi_options', $parameters_array );
}

add_action( 'wp_enqueue_scripts', 'jpibfi_add_plugin_scripts' );

function jpibfi_print_header_style_action() {
	if ( ! ( jpibfi_add_plugin() ) )
		return;

	$options = get_option( JPIBFI_VISUAL_OPTIONS );

	$button_css = $options[ 'button_css' ];
	$use_custom_image = isset( $options[ 'use_custom_image' ] ) && $options[ 'use_custom_image' ] == "1";

	$width  = $use_custom_image ? $options['custom_image_width'] : JPIBFI_IMAGE_WIDTH;
	$height = $use_custom_image ? $options['custom_image_height'] : JPIBFI_IMAGE_HEIGHT;
	$url    = $use_custom_image ? $options['custom_image_url'] : JPIBFI_IMAGE_URL;

	$transHex = "#" . jpibfi_convert_to_hex( $options['transparency_value'] ) . "ffffff";

	?>
	<!--[if lt IE 9]>
		<style type="text/css">
			.pinit-overlay {
				background-image: url( '<?php echo plugins_url( '/images/transparency_0.png', __FILE__ ); ?>' );
			}
			.pinit-overlay:hover {
				filter:progid:DXImageTransform.Microsoft.gradient(startColorstr='<?php echo $transHex; ?>',endColorstr='<?php echo $transHex; ?>');
			}
		</style>
	<![endif]-->

<style type="text/css">
	.pinit-overlay:hover {
		background-color: rgba(255, 255, 255, <?php echo $options['transparency_value']; ?>);
	}
	.pinit-overlay a {
		width: <?php echo $width; ?>px;
		height: <?php echo $height; ?>px;
		background: transparent url('<?php echo $url; ?>') no-repeat 0 0;
		<?php echo $button_css; ?>
	}
</style>
<?php
}

add_action( 'wp_head', 'jpibfi_print_header_style_action' );

/*
 * Adds a hidden field with url and and description of the pin that's used when user uses "Link to individual page"
 * Thanks to go brocheafoin, who added most of the code that handles creating description
 */
function jpibfi_print_hidden_field_script( $content ) {
	if ( ! jpibfi_add_plugin() )
		return $content;
	global $post;

	$options = get_option( JPIBFI_VISUAL_OPTIONS );

	$add_attributes = false == is_singular() && isset( $options[ 'use_post_url' ] ) && '1' == $options[ 'use_post_url' ];

	$attributes_html = '';

	//if we need to add additional attributes to handle use_post_url setting
	if ( $add_attributes ){
		//if page description should be used as pin description and an excerpt for the post exists
		if ( has_excerpt( $post->ID ) && 2 == $options[ 'description_option' ] )
			$description = wp_kses( $post->post_excerpt, array() );
		else
			$description = get_the_title($post->ID);

		$attributes_html .= 'data-jpibfi-url="' . get_permalink( $post->ID ) . '" ' ;
		$attributes_html .= 'data-jpibfi-description ="' . esc_attr( $description ) . '" ';
	}


	$input_html = '<input class="jpibfi" type="hidden" ' . $attributes_html . '>';
	return $input_html . $content;
}

add_filter( "the_content", 'jpibfi_print_hidden_field_script' );

/*
 *
 * POST EDITOR CODE
 *
 */

function jpibfi_add_meta_box() {
	//for posts
	add_meta_box(
		'jpibfi_settings_id', // this is HTML id of the box on edit screen
		'jQuery Pin It Button for Images - ' . __( 'Settings', 'jpibfi' ), // title of the box
		'jpibfi_print_meta_box', // function to be called to display the checkboxes, see the function below
		'post', // on which edit screen the box should appear
		'side', // part of page where the box should appear
		'default' // priority of the box
	);

	//for pages
	add_meta_box(
		'jpibfi_settings_id',
			'jQuery Pin It Button for Images - ' . __( 'Settings', 'jpibfi' ),
		'jpibfi_print_meta_box',
		'page',
		'side',
		'default'
	);
}

add_action( 'add_meta_boxes', 'jpibfi_add_meta_box' );

// display the metabox
function jpibfi_print_meta_box( $post, $metabox ) {

	wp_nonce_field( plugin_basename( __FILE__ ), 'jpibfi_nonce' );

	$post_meta = get_post_meta( $post->ID, JPIBFI_METADATA, true );
	$checked = isset( $post_meta ) && isset( $post_meta['jpibfi_disable_for_post'] ) && $post_meta['jpibfi_disable_for_post'] == '1';

	echo '<input type="checkbox" id="jpibfi_disable_for_post" name="jpibfi_disable_for_post" value="1"'	. checked( $checked, true, false ) . '>';
	echo '<label for="jpibfi_disable_for_post">' . __( 'Disable "Pin it" button for this post (works only on single pages/posts)', 'jpibfi' ) . '</label><br />';
}

function jpibfi_save_meta_data( $post_id ) {

	//check user's permissions
	if ( ! current_user_can( 'edit_post', $post_id ) )
		return;
	// check if this isn't an auto save
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
		return;
	// security check = updating possible only using post edit form
	if ( !isset( $_POST['jpibfi_nonce'] ) || ! wp_verify_nonce( $_POST['jpibfi_nonce'], plugin_basename( __FILE__ ) ) )
		return;


	$post_meta = array( 'jpibfi_disable_for_post' => '0' );
	// now store data in custom fields based on checkboxes selected
	$post_meta['jpibfi_disable_for_post'] =
			isset( $_POST['jpibfi_disable_for_post'] ) && $_POST['jpibfi_disable_for_post'] == '1' ? '1' : '0';

	if ( $post_meta['jpibfi_disable_for_post'] == '1' )
		update_post_meta( $post_id, JPIBFI_METADATA, $post_meta );
	else
		delete_post_meta( $post_id, JPIBFI_METADATA );
}

add_action( 'save_post', 'jpibfi_save_meta_data' );

//END POST EDITOR CODE

//Delete everything the plugin added into DB
function jpibfi_uninstall_plugin() {

	//delete all added options
	delete_option( JPIBFI_SELECTION_OPTIONS );
	delete_option( JPIBFI_VISUAL_OPTIONS );
	delete_option( JPIBFI_VERSION_OPTION );

	//delete added metadata from all posts
	delete_post_meta_by_key( JPIBFI_METADATA );
}

register_uninstall_hook( __FILE__, 'jpibfi_uninstall_plugin' );

/*
 *
 * ADMIN SETTINGS
 *
 */

/*
 * Function updates DB if it detects new version of the plugin
 */
function jpibfi_update_plugin() {


	$version = get_option( JPIBFI_VERSION_OPTION );

	//if the plugin isn't installed at all or the version is below 1.00
	if ( false == $version ) {
		//rewrite old settings (if they exist) to new settings
		jpibfi_convert_settings();

		//if new settings are still absent (plugin wasn't installed previously), we need to add them
		if( false == get_option( JPIBFI_VISUAL_OPTIONS ) )
			add_option( JPIBFI_VISUAL_OPTIONS, jpibfi_default_visual_options() );

		if( false == get_option( JPIBFI_SELECTION_OPTIONS ) )
			add_option( JPIBFI_SELECTION_OPTIONS, jpibfi_default_selection_options() );

	}	else if ( (float)$version < (float)JPIBFI_VERSION ) { //if the plugins version is older than current, we need to update options with new defaults

		$option = get_option( JPIBFI_VISUAL_OPTIONS );
		jpibfi_update_option_fields( $option, jpibfi_default_visual_options(), JPIBFI_VISUAL_OPTIONS );

		$option = get_option( JPIBFI_SELECTION_OPTIONS );
		jpibfi_update_option_fields( $option, jpibfi_default_selection_options(), JPIBFI_SELECTION_OPTIONS );
	}

	//update the version of the plugin stored in option
	update_option( JPIBFI_VERSION_OPTION, JPIBFI_VERSION );
}

register_activation_hook( __FILE__, 'jpibfi_update_plugin' );
add_action( 'plugins_loaded', 'jpibfi_update_plugin' );

/*
 * This function converts old settings from versions earlier than 1.00 (if there are any) format to new settings format (changed names of options, etc)
 */
function jpibfi_convert_settings() {
	//get all old options
	$basic_option = get_option( "jptbfi_options" );
	$advanced_options = get_option( 'jptbfi_advanced_options' );
	$button_css = get_option( 'jpibfi_button_custom_css' );

	//if options don't exist, there's nothing to convert so the function ends
	if ( false == $basic_option || false == $advanced_options )
		return;

	//rewrite them to new options
	$default_selection_options = jpibfi_default_selection_options();

	$selection_options = array();
	$selection_options[ 'image_selector' ] = isset ( $basic_option[ 'image_selector' ] ) ? $basic_option[ 'image_selector' ] : $default_selection_options[ 'image_selector' ];
	$selection_options[ 'disabled_classes' ] = isset ( $basic_option[ 'disabled_classes' ] ) ? $basic_option[ 'disabled_classes' ] : $default_selection_options[ 'disabled_classes' ];
	$selection_options[ 'enabled_classes' ] = isset ( $basic_option[ 'enabled_classes' ] ) ? $basic_option[ 'enabled_classes' ] : $default_selection_options[ 'enabled_classes' ];
	$selection_options[ 'min_image_height' ] = isset ( $basic_option[ 'min_image_height' ] ) ? $basic_option[ 'min_image_height' ] : $default_selection_options[ 'min_image_height' ];
	$selection_options[ 'min_image_width' ] = isset ( $basic_option[ 'min_image_width' ] ) ? $basic_option[ 'min_image_width' ] : $default_selection_options[ 'min_image_width' ];

	$selection_options[ 'show_on_home' ] = isset ( $advanced_options[ 'on_home' ] ) && '1' == $advanced_options[ 'on_home' ] ? '1' : '0';
	$selection_options[ 'show_on_single' ] = isset ( $advanced_options[ 'on_single' ] ) && '1' == $advanced_options[ 'on_single' ] ? '1' : '0';
	$selection_options[ 'show_on_page' ] = isset ( $advanced_options[ 'on_page' ] ) && '1' == $advanced_options[ 'on_page' ] ? '1' : '0';
	$selection_options[ 'show_on_category' ] = isset ( $advanced_options[ 'on_category' ] )&& '1' == $advanced_options[ 'on_category' ] ? '1' : '0';
	$selection_options[ 'show_on_blog' ] = isset ( $advanced_options[ 'on_blog' ] ) && '1' == $advanced_options[ 'on_blog' ] ? '1' : '0';

	update_option( JPIBFI_SELECTION_OPTIONS, $selection_options );
	//just in case any new fields are added to the option in the future
	jpibfi_update_option_fields( $selection_options, jpibfi_default_selection_options(), JPIBFI_SELECTION_OPTIONS );

	//visual options
	$default_visual_options = jpibfi_default_visual_options();

	$visual_options = array();
	$visual_options[ 'description_option' ] = isset ( $basic_option[ 'description_option' ] ) ? $basic_option[ 'description_option' ] : $default_visual_options[ 'description_option' ];
	$visual_options[ 'transparency_value' ] = isset ( $basic_option[ 'transparency_value' ] ) ? $basic_option[ 'transparency_value' ] : $default_visual_options[ 'transparency_value' ];
	$visual_options[ 'use_custom_image' ] = isset ( $basic_option[ 'use_custom_image' ] ) && '1' == $basic_option[ 'use_custom_image' ] ? '1' : '0';
	$visual_options[ 'custom_image_url' ] = isset ( $basic_option[ 'custom_image_url' ] ) ? $basic_option[ 'custom_image_url' ] : $default_visual_options[ 'custom_image_url' ];
	$visual_options[ 'custom_image_height' ] = isset ( $basic_option[ 'custom_image_height' ] ) ? $basic_option[ 'custom_image_height' ] : $default_visual_options[ 'custom_image_height' ];
	$visual_options[ 'custom_image_width' ] = isset ( $basic_option[ 'custom_image_width' ] ) ? $basic_option[ 'custom_image_width' ] : $default_visual_options[ 'custom_image_width' ];
	$visual_options[ 'use_post_url' ] = isset ( $basic_option[ 'use_post_url' ] ) && '1' == $basic_option[ 'use_post_url' ] ? '1' : '0';
	$visual_options[ 'button_position' ] = isset ( $basic_option[ 'button_position' ] ) ? $basic_option[ 'button_position' ] : $default_visual_options[ 'button_position' ];
	$visual_options[ 'button_css' ] = true == $button_css ? $button_css : $default_visual_options[ 'button_css' ];

	update_option( JPIBFI_VISUAL_OPTIONS, $visual_options );
	//just in case any new fields are added to the option in the future
	jpibfi_update_option_fields( $visual_options, jpibfi_default_visual_options(), JPIBFI_VISUAL_OPTIONS );

	//delete all old options
	delete_option( "jptbfi_options" );
	delete_option( 'jptbfi_advanced_options' );
	delete_option( 'jpibfi_button_custom_css' );
}

function jpibfi_print_admin_page_action() {
	$page = add_submenu_page(
		'options-general.php',
		'jQuery Pin It Button For Images', 	// The value used to populate the browser's title bar when the menu page is active
		'jQuery Pin It Button For Images', // The text of the menu in the administrator's sidebar
		'administrator',					// What roles are able to access the menu
		'jpibfi_settings',				// The ID used to bind submenu items to this menu
		'jpibfi_print_admin_page'				// The callback function used to render this menu
	);

	add_action( 'admin_print_styles-' . $page, 'jpibfi_add_admin_site_scripts' );
}

add_action( 'admin_menu', 'jpibfi_print_admin_page_action' );

/*
 * adds admin scripts
 */
function jpibfi_add_admin_site_scripts() {
	$jpibfi_button_css = jpibfi_button_css();

	wp_register_style( 'jquery-pin-it-button-admin-style', plugins_url( '/css/admin.css', __FILE__ ), array(), JPIBFI_VERSION, 'all' );
	wp_enqueue_style( 'jquery-pin-it-button-admin-style' );

	wp_enqueue_script( 'jquery-pin-it-button-admin-script', plugins_url( '/js/admin.js', __FILE__ ), array( 'jquery' ), JPIBFI_VERSION, false );
	wp_localize_script( 'jquery-pin-it-button-admin-script', 'jpibfi_button_css', $jpibfi_button_css );

	if ( function_exists( "wp_enqueue_media") ) {
		wp_enqueue_media();
		wp_enqueue_script( 'jpibfi-upload-new', plugins_url( '/js/upload-button-new.js', __FILE__ ), array(), JPIBFI_VERSION, false );
	} 	else {
		wp_enqueue_script( 'jpibfi-upload-old', plugins_url( '/js/upload-button-old.js', __FILE__ ), array('thickbox', 'media-upload' ), JPIBFI_VERSION, false );
	}
}


function jpibfi_plugin_settings_filter( $links ) {
	$settings_link = '<a href="options-general.php?page=jpibfi_settings">' . __( 'Settings', 'jpibfi' ) . '</a>';
	array_unshift( $links, $settings_link );
	return $links;
}

$jpibfi_plugin = plugin_basename( __FILE__ );
add_filter( "plugin_action_links_$jpibfi_plugin", 'jpibfi_plugin_settings_filter' );

function jpibfi_print_admin_page() {
	?>
	<!-- Create a header in the default WordPress 'wrap' container -->
	<div class="wrap">

		<h2><?php _e( 'jQuery Pin It Button For Images Options', 'jpibfi' ); ?></h2>

		<?php
		$tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'selection_options';
		if ( 'selection_options' != $tab && 'visual_options' != $tab )
			$tab = 'selection_options';
		?>
		<div id="icon-plugins" class="icon32"></div>
		<h2 class="nav-tab-wrapper">
			<a href="?page=jpibfi_settings&tab=selection_options" class="nav-tab <?php echo $tab == 'selection_options' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Selection Options', 'jpibfi' ); ?></a>
			<a href="?page=jpibfi_settings&tab=visual_options" class="nav-tab <?php echo $tab == 'visual_options' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Visual Options', 'jpibfi' ); ?></a>
		</h2>

		<p>
			If you like the plugin and would like to support its development, please <a href="http://bit.ly/Uw2mEP" target="_blank" rel="nofollow"><b>donate</b></a>.
		</p>
		<form method="post" action="options.php">
			<?php

			if ( 'selection_options' == $tab ) {
				settings_fields( 'jpibfi_selection_options' );
				do_settings_sections( 'jpibfi_selection_options' );
			} else if ( 'visual_options' == $tab) {
				settings_fields( 'jpibfi_visual_options' );
				do_settings_sections( 'jpibfi_visual_options' );
			}

			submit_button();
			?>
		</form>

	</div>
<?php

	//cumbersome, but needed for error management to work properly in WP 3.3
	delete_option( JPIBFI_SELECTION_OPTIONS . '_errors' );
	delete_option( JPIBFI_VISUAL_OPTIONS . '_errors' );
}

/*
 *
 * SELECTION OPTIONS
 *
 */

/*
 * Default values for selection options section
 */
function jpibfi_default_selection_options() {
	$defaults = array(
		'image_selector'      => 'div.jpibfi_container img',
		'disabled_classes'    => 'nopin;wp-smiley',
		'enabled_classes'     => '',
		'min_image_height'		=> '0',
		'min_image_width'			=> '0',
		'show_on_home'     		=> '1',
		'show_on_single'   		=> '1',
		'show_on_page'    		=> '1',
		'show_on_category' 		=> '1',
		'show_on_blog'				=> '1'
	);

	return $defaults;
}

/*
 * Defines selection options section and adds all required fields
 */
function jpibfi_initialize_selection_options() {

	// First, we register a section.
	add_settings_section(
		'selection_options_section',			// ID used to identify this section and with which to register options
		__( 'Selection', 'jpibfi' ),		// Title to be displayed on the administration page
		'jpibfi_selection_options_callback',	// Callback used to render the description of the section
		'jpibfi_selection_options'		// Page on which to add this section of options
	);

	//lThen add all necessary fields to the section
	add_settings_field(
		'image_selector',						// ID used to identify the field throughout the plugin
		__( 'Image selector', 'jpibfi' ),							// The label to the left of the option interface element
		'jpibfi_image_selector_callback',	// The name of the function responsible for rendering the option interface
		'jpibfi_selection_options',	// The page on which this option will be displayed
		'selection_options_section',			// The name of the section to which this field belongs
		array(								// The array of arguments to pass to the callback. In this case, just a description.
			__( 'jQuery selector for all the images that should have the "Pin it" button. Set
						the value to <a href="#" class="jpibfi_selector_option">div.jpibfi_container img</a> if you want
						the "Pin it" button to appear only on images in content or <a href="#" class="jpibfi_selector_option">img</a>
						to appear on all images on site (including sidebar, header and footer). If you know a thing or two about jQuery, you might use your own selector.
						<a href="http://api.jquery.com/category/selectors/" target="_blank">Click here</a> to read
						about jQuery selectors.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'disabled_classes',
		__( 'Disabled classes', 'jpibfi' ),
		'jpibfi_classes_callback',
		'jpibfi_selection_options',
		'selection_options_section',
		array(
			'disabled_classes',
			__( 'Pictures with these CSS classes won\'t show the "Pin it" button. Please separate multiple classes with semicolons. Spaces are not accepted.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'enabled_classes',
		__( 'Enabled classes', 'jpibfi' ),
		'jpibfi_classes_callback',
		'jpibfi_selection_options',
		'selection_options_section',
		array(
			'enabled_classes',
			__( 'Only pictures with these CSS classes will show the "Pin it" button. Please
						separate multiple classes with semicolons. If this field is empty, images with any (besides
						disabled ones) classes will show the Pin It button.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'show_on_field',
		__( 'On which pages the "Pin it" button should be shown', 'jpibfi' ),
		'jpibfi_show_on_field_callback',
		'jpibfi_selection_options',
		'selection_options_section',
		array(
			__( 'Check on which pages you want the Pinterest button to show up.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'min_image',
		__( 'Minimum resolution that should trigger the "Pin it" button to show up', 'jpibfi' ),
		'jpibfi_min_image_callback',
		'jpibfi_selection_options',
		'selection_options_section',
		array(
			__( 'If you\'d like the "Pin it" button to not show up on small images (e.g. social media icons), just set the appropriate values above. The default value (0,0) causes the "Pin it" button to show on every eligible image.', 'jpibfi' ),
		)
	);

	register_setting(
		'jpibfi_selection_options',
		'jpibfi_selection_options',
		'jpibfi_sanitize_selection_options'
	);
}

add_action( 'admin_init', 'jpibfi_initialize_selection_options' );

function jpibfi_selection_options_callback() {
	echo '<p>' . __('Which images can be pinned', 'jpibfi') . '</p>';
}

function jpibfi_image_selector_callback( $args ) {

	$options = jpibfi_get_selection_options();

	$selector = esc_attr( $options['image_selector'] );

	echo '<input type="text" id="image_selector" name="jpibfi_selection_options[image_selector]" value="' . $selector . '"/>';
	echo jpibfi_create_description( $args[0] );
}

function jpibfi_classes_callback( $args ) {

	$options = jpibfi_get_selection_options();

	$value = esc_attr( $options[ $args[0] ] );

	echo '<textarea id="' . $args[0] . '" name="jpibfi_selection_options[' . $args[0] . ']">' . $value . '</textarea>';
	echo jpibfi_create_description( $args[1] );
	echo jpibfi_create_errors( $args[0] );
}

function jpibfi_show_on_field_callback( $args ) {
	$options = jpibfi_get_selection_options();

	$show_on_home = jpibfi_exists_and_equal_to( $options, 'show_on_home', '1' );
	$show_on_page = jpibfi_exists_and_equal_to( $options, 'show_on_page', '1' );
	$show_on_single = jpibfi_exists_and_equal_to( $options, 'show_on_single', '1' );
	$show_on_category = jpibfi_exists_and_equal_to( $options, 'show_on_category', '1' );
	$show_on_blog = jpibfi_exists_and_equal_to( $options, 'show_on_blog', '1' );
	?>

	<input type="checkbox" id="show_on_home" name="jpibfi_selection_options[show_on_home]" <?php checked( true, $show_on_home ); ?> value="1" />
	<label for="show_on_home"><?php _e( 'Home page', 'jpibfi' ); ?></label><br/>
	<input type="checkbox" id="show_on_page" name="jpibfi_selection_options[show_on_page]" <?php checked( true, $show_on_page ); ?> value="1" />
	<label for="show_on_page"><?php _e( 'Pages', 'jpibfi' ); ?></label><br />
	<input type="checkbox" id="show_on_single" name="jpibfi_selection_options[show_on_single]" <?php checked( true, $show_on_single ); ?> value="1" />
	<label for="show_on_single"><?php _e( 'Single posts', 'jpibfi' ); ?></label><br />
	<input type="checkbox" id="show_on_category"	name="jpibfi_selection_options[show_on_category]" <?php checked( true, $show_on_category ); ?> value="1" />
	<label for="show_on_category"><?php _e( 'Category and archive pages', 'jpibfi' ); ?></label><br />
	<input type="checkbox" id="show_on_blog"	name="jpibfi_selection_options[show_on_blog]" <?php checked( true, $show_on_blog ); ?> value="1" />
	<label for="show_on_blog"><?php _e( 'Blog pages', 'jpibfi' ); ?></label>

	<?php
	echo jpibfi_create_description( $args[0] );
}

function jpibfi_min_image_callback( $args ) {
	$options = jpibfi_get_selection_options();

	$min_image_height = $options[ 'min_image_height' ];
	$min_image_width = $options[ 'min_image_width' ];
	?>

	<p>
		<label for="min_image_height"><?php _e('Height', 'jpibfi'); ?></label>
		<input type="number" min="0" step="1" id="min_image_height" name="jpibfi_selection_options[min_image_height]" value="<?php echo $min_image_height; ?>"
					 class="small-text" /> px
		<?php echo jpibfi_create_errors( 'min_image_height' ); ?>
	</p>

	<p>
		<label for="min_image_width"><?php _e('Width', 'jpibfi'); ?></label>
		<input type="number" min="0" step="1" id="min_image_width" name="jpibfi_selection_options[min_image_width]" value="<?php echo $min_image_width; ?>"
					 class="small-text" /> px
		<?php echo jpibfi_create_errors( 'min_image_width' ); ?>
	</p>

	<?php

	echo jpibfi_create_description( $args[0] );
}

function jpibfi_sanitize_selection_options( $input ) {

	foreach( $input as $key => $value ) {
		switch($key) {
			case 'disabled_classes':
			case 'enabled_classes':
				if ( false == jpibfi_contains_css_class_names_or_empty( $input [ $key ] ) ) {

					$field = '';
					if ( 'disabled_classes' == $key )
						$field = __( 'Disabled classes', 'jpibfi' );
					else if ( 'enabled_classes' == $key )
						$field = __( 'Enabled classes', 'jpibfi' );

					add_settings_error(
						$key,
						esc_attr( 'settings_updated' ),
						$field . ' - ' . __('the given value doesn\'t meet the requirements. Please correct it and try again.', 'jpibfi')
					);
				}
				break;
			case 'min_image_height':
			case 'min_image_width':
				if ( !is_numeric( $value ) || $value < 0 ) {

					$field = '';
					if ( 'min_image_height' == $key )
						$field = __( 'Minimum image height', 'jpibfi' );
					else if ( 'min_image_width' == $key )
						$field = __( 'Minimum image width', 'jpibfi' );

					add_settings_error(
						$key,
						esc_attr( 'settings_updated' ),
						$field . ' - ' . __('value must be a number greater or equal to 0.', 'jpibfi')
					);
				}
				break;
		}

	}

	$errors = get_settings_errors();

	if ( count( $errors ) > 0 ) {

		update_option( JPIBFI_SELECTION_OPTIONS . '_errors', $input );
		return get_option( JPIBFI_SELECTION_OPTIONS );

	} else {

		delete_option( JPIBFI_SELECTION_OPTIONS . '_errors' );
		return $input;

	}
}

function jpibfi_get_selection_options() {
	return jpibfi_get_options( JPIBFI_SELECTION_OPTIONS );
}

/*
 *
 * VISUAL OPTIONS
 *
 */

/*
 * Default values for visual options section
 */
function jpibfi_default_visual_options() {
	$jpibfi_button_css = jpibfi_button_css();

	$defaults = array(
		'transparency_value'  => '0.5',
		'description_option'  => '1',
		'use_custom_image'    => '0',
		'custom_image_url'    => '',
		'custom_image_height' => '0',
		'custom_image_width'  => '0',
		'use_post_url'        => '0',
		'button_position'			=> '0',
		'button_css'					=> $jpibfi_button_css[0]
	);

	return $defaults;
}

/*
 * Defines visual options section and defines all required fields
 */
function jpibfi_initialize_visual_options() {

	// First, we register a section.
	add_settings_section(
		'visual_options_section',			// ID used to identify this section and with which to register options
		__( 'Visual', 'jpibfi' ),		// Title to be displayed on the administration page
		'jpibfi_visual_options_callback',	// Callback used to render the description of the section
		'jpibfi_visual_options'		// Page on which to add this section of options
	);

	//Then add all necessary fields to the section
	add_settings_field(
		'description_option',
		__( 'Description source', 'jpibfi' ),
		'jpibfi_description_option_callback',
		'jpibfi_visual_options',
		'visual_options_section',
		array(
			__( 'From where the Pinterest message should be taken.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'use_post_url',
		__( 'Linked page', 'jpibfi' ),
		'jpibfi_use_post_url_callback',
		'jpibfi_visual_options',
		'visual_options_section',
		array(
			__( 'When checked, the link on Pinterest will always point to the individual page with the image and title of this individual page will be used if you\'ve selected Title as the description source, even when the image was pinned on an archive page, category page or homepage. If false, the link will point to the URL the user is currently on.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'transparency_value',
		__( 'Transparency value', 'jpibfi' ),
		'jpibfi_transparency_value_callback',
		'jpibfi_visual_options',
		'visual_options_section',
		array(
			__( 'This setting sets the transparency of the layer covering the image.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'custom_pin_it_button',
		__( 'Custom "Pin It" button', 'jpibfi' ),
		'jpibfi_custom_pin_it_button_callback',
		'jpibfi_visual_options',
		'visual_options_section',
		array(
			__( 'Check the <b>Use custom image</b> checkbox, specify image\'s URL, height and width to use your own Pinterest button design. You can just upload an image using Wordpress media library if you wish.', 'jpibfi' ),
		)
	);

	add_settings_field(
		'pin_it_button_position',
		__( '"Pin it" button position', 'jpibfi' ),
		'jpibfi_pin_it_button_position_callback',
		'jpibfi_visual_options',
		'visual_options_section',
		array(
			__( 'Where the "Pin it" button should appear on the image.', 'jpibfi' ),
		)
	);

	register_setting(
		'jpibfi_visual_options',
		'jpibfi_visual_options',
		'jpibfi_sanitize_visual_options'
	);

}

add_action( 'admin_init', 'jpibfi_initialize_visual_options' );

function jpibfi_visual_options_callback() {
	echo '<p>' . __('How it should look like', 'jpibfi') . '</p>';
}

function jpibfi_description_option_callback( $args ) {
	$options = jpibfi_get_visual_options();


	$description_option = $options[ 'description_option' ];
	?>

	<select id="description_option" name="jpibfi_visual_options[description_option]">
		<option value="1" <?php selected ( "1", $description_option ); ?>><?php _e( 'Page title', 'jpibfi' ); ?></option>
		<option value="2" <?php selected ( "2", $description_option ); ?>><?php _e( 'Page description', 'jpibfi' ); ?></option>
		<option value="3" <?php selected ( "3", $description_option ); ?>><?php _e( 'Picture title or (if title not available) alt attribute', 'jpibfi' ); ?></option>
		<option value="4" <?php selected ( "4", $description_option ); ?>><?php _e( 'Site title (Settings->General)', 'jpibfi' ); ?></option>
	</select>

	<?php
	echo jpibfi_create_description( $args[0] );
}

function jpibfi_use_post_url_callback( $args ) {

	$options = jpibfi_get_visual_options();
	$use_post_url = jpibfi_exists_and_equal_to( $options, 'use_post_url', '1' );

	echo '<input type="checkbox" id="use_post_url" name="jpibfi_visual_options[use_post_url]" value="1" ' . checked( "1", $use_post_url, false ) . '>';
	echo '<label for="use_post_url">' . __( 'Always link to individual post page', 'jpibfi' ) . '</label>';

	echo jpibfi_create_description( $args[0] );
}

function jpibfi_transparency_value_callback( $args ) {
	$options = jpibfi_get_visual_options();

	$transparency_value = $options[ 'transparency_value' ];

	echo '<label for="transparency_value">' . _e('Choose transparency (between 0.00 and 1.00)', 'jpibfi') . '</label><br/>';
	echo '<input type="number" min="0" max="1" step="0.01" id="transparency_value" name="jpibfi_visual_options[transparency_value]"' .
			'value="' . $transparency_value . '" class="small-text" >';
	echo jpibfi_create_description( $args[0] );
	echo jpibfi_create_errors( 'transparency_value' );
}

function jpibfi_custom_pin_it_button_callback( $args ) {
	$options = jpibfi_get_visual_options();

	$use_custom_image = jpibfi_exists_and_equal_to( $options, 'use_custom_image', '1' );
	$custom_image_url = $options[ 'custom_image_url' ];
	$custom_image_height = $options[ 'custom_image_height' ];
	$custom_image_width = $options[ 'custom_image_width' ];

	?>
	<p>
		<input type="checkbox" id="use_custom_image" name="jpibfi_visual_options[use_custom_image]" value="1" <?php checked( true, $use_custom_image ); ?> >
		<label class="chbox-label" for="use_custom_image"><?php _e( 'Use custom image', 'jpibfi' ); ?></label>
	</p>

	<button id=upload-image><?php _e( 'Upload an image using media library','jpibfi' ); ?></button><br />

	<p>
		<label for="custom_image_url"><?php _e( 'URL address of the image', 'jpibfi' ); ?></label>
		<input type="url" id="custom_image_url" name="jpibfi_visual_options[custom_image_url]" value="<?php echo $custom_image_url; ?>">
	</p>

	<p>
		<label for="custom_image_height"><?php _e( 'Height', 'jpibfi' ); ?></label>
		<input type="number" min="0" step="1" id="custom_image_height" name="jpibfi_visual_options[custom_image_height]" value="<?php echo $custom_image_height; ?>"
					 class="small-text" /> px
		<?php echo jpibfi_create_errors( 'custom_image_height' ); ?>
	</p>

	<p>
		<label for="custom_image_width"><?php _e( 'Width', 'jpibfi' ); ?></label>
		<input type="number" min="0" step="1" id="custom_image_width" name="jpibfi_visual_options[custom_image_width]" value="<?php echo $custom_image_width; ?>"
					 class="small-text"  /> px
		<?php echo jpibfi_create_errors( 'custom_image_width' ); ?>
	</p>

	<p>
		<b><?php _e( 'Custom Pin It button preview', 'jpibfi' ); ?></b><br/>
		<span id="custom_button_preview" style="width: <?php echo $custom_image_width; ?>px; height: <?php echo $custom_image_height; ?>px; background-image: url('<?php echo $custom_image_url; ?>');">
			Preview
		</span><br/>
		<button id="refresh_custom_button_preview"><?php _e( 'Refresh preview', 'jpibfi' ); ?></button>
	</p>

	<?php

	echo jpibfi_create_description( $args[0] );
}

function jpibfi_pin_it_button_position_callback( $args ) {
	$options = jpibfi_get_visual_options();

	$jpibfi_button_dropdown = array(
		__( 'Top left (default)', 'jpibfi' ),
		__( 'Top right', 'jpibfi' ),
		__( 'Bottom left', 'jpibfi' ),
		__( 'Bottom right', 'jpibfi' ),
		__( 'Middle', 'jpibfi' ),
		__( 'Custom', 'jpibfi' )
	);

	$button_position = $options[ 'button_position' ];
	$button_css = $options[ 'button_css' ];

	?>

	<select name="jpibfi_visual_options[button_position]" id="button_position">
		<?php for( $i = 0; $i < count( $jpibfi_button_dropdown ); $i++) { ?>
			<option value="<?php echo $i; ?>" <?php selected( $i, $button_position ); ?>><?php echo $jpibfi_button_dropdown[ $i ]; ?></option>
		<?php } ?>
	</select><br/>
	<textarea id="button_css" rows="5" cols="50" name="jpibfi_visual_options[button_css]"><?php echo $button_css; ?></textarea>

	<?php
	echo jpibfi_create_description( $args[0] );
}

function jpibfi_sanitize_visual_options( $input ) {
	$jpibfi_button_css = jpibfi_button_css();

	foreach( $input as $key => $value ) {

		switch($key) {
			case 'transparency_value':
				if ( !is_numeric( $input[ $key ] ) || ( $input[ $key ] < 0.0 ) || ( $input[ $key ] > 1.0 ) ) {

					add_settings_error(
						$key,
						esc_attr( 'settings_updated' ),
						__('Transparency value must be a number between 0.0 and 1.0.', 'jpibfi')
					);
				}
				break;
			case 'custom_image_height':
			case 'custom_image_width':
				$name = "";
				if ( 'custom_image_height' == $key )
					$name = __('Custom image height', 'jpibfi' );
				else if ( 'custom_image_width' == $key )
					$name = __('Custom image width', 'jpibfi' );

				if ( '' != $value && ( !is_numeric( $value ) || $value < 0 ) ) {
					add_settings_error(
						$key,
						esc_attr( 'settings_updated' ),
						$name . __(' must be a number greater or equal to 0.', 'jpibfi')
					);
				}
				break;
		}
	}

	//if user didn't specify his custom css, we need to save button_css setting as one of the defaults
	if ( '5' != $input[ 'button_position' ] )
		$input[ 'button_css' ] = $jpibfi_button_css[ $input[ 'button_position' ] ];

	$errors = get_settings_errors();

	if ( count( $errors ) > 0 ) {

		update_option( JPIBFI_VISUAL_OPTIONS . '_errors', $input );
		return get_option( JPIBFI_VISUAL_OPTIONS );

	} else {

		delete_option( JPIBFI_VISUAL_OPTIONS . '_errors' );
		return $input;

	}
}

function jpibfi_get_visual_options() {
	return jpibfi_get_options( JPIBFI_VISUAL_OPTIONS );
}

/*
 *
 * UTILITIES
 *
 */

/*
 * Function returns properly formatted description of a setting
 */
function jpibfi_create_description( $desc ) {
	return '<p class="description">' . $desc . '</p>';
}

function jpibfi_create_errors( $setting_name ) {
	$error = "";

	$errors_array = get_settings_errors( $setting_name );

	if ( count ( $errors_array ) > 0 ) {
		$error .= '<div class="jpibfi-error-message">';
		for( $i = 0; $i < count( $errors_array ); $i++ ){
			$error .= $errors_array[ $i ]['message'] . '</br>';
		}
		$error .= '</div>';
	}

	return $error;
}

/*
 * writes or returns chosen text depending whether the given condition is true or false
 */
function jpibfi_conditional( $condition, $true_value = "", $false_value = "", $echo = true) {
	if ( $condition ) {
		if ( $echo )
			echo $true_value;
		return $true_value;
	}
	if ( $echo )
		echo $false_value;
	return $false_value;
}

function jpibfi_exists_and_equal_to( $array , $name, $expected_value = "1" ) {
	return isset( $array[ $name ] ) && $array[ $name ] == $expected_value;
}

/*
 * If there are errors, let's return the array with 'unsaved' data
 * 'Saved' settings otherwise
 */
function jpibfi_get_options( $name ) {
	//cumbersome, but works in WP 3.3
	$options = get_option( $name . '_errors' );

	return false == $options ? get_option( $name ) : $options;

	//the code below is a much better error management solution, but doesn't work in WP 3.3
//	if ( JPIBFI_SELECTION_OPTIONS == $name )
//		$settings_list = jpibfi_default_selection_options();
//	else
//		$settings_list = jpibfi_default_visual_options();
//
//	$errors = false;
//
//	foreach( $settings_list as $key => $value) {
//		if ( count( get_settings_errors( $key ) ) > 0 ) {
//			$errors = true;
//			break;
//		}
//	}
//	return $errors ? get_option( $name . '_errors' ) : get_option( $name );
}

/*
 * Function makes sure that option has all needed fields by checking with defaults
 */
function jpibfi_update_option_fields( $option, $default_option, $option_name ) {

	$option_changed = false;

	foreach($default_option as $key => $value) {
		if ( false == array_key_exists( $key, $option ) ) {
			$options [ $key ] = $value;
			$option_changed = true;
		}
	}

	if ( $option_changed )
		update_option( $option_name, $option );

}

/*
 * function converts 0.0..1.0 value to 00..FF
 */
function jpibfi_convert_to_hex( $val ) {

	$transparencyDec = floatval( $val );
	$transparencyDec = (int)( $transparencyDec * 255 );
	$transparencyHex = dechex( (int) $transparencyDec );
	if ( strlen( $transparencyHex) == 1 )
		$transparencyHex = "0" . $transparencyHex;

	return $transparencyHex;
}

function jpibfi_is_string_css_class_name( $class_name ) {
	return 1 == preg_match( "/^-?[_a-zA-Z]+[_a-zA-Z0-9-]*$/", $class_name );
}

//checks if given string contains class names separated by semicolons or is empty
function jpibfi_contains_css_class_names_or_empty( $str ) {
	if ( 0 == strlen( $str ) )
		return true;

	$names = explode( ';', $str );
	$only_class_names = true;

	for( $i = 0; $i < count( $names ) && $only_class_names; $i++ )
		$only_class_names = jpibfi_is_string_css_class_name( $names [ $i ] );

	return $only_class_names;
}

function jpibfi_button_css() {
	$css = array();
	$css[0] = esc_html(	"top:0%;&#10;left:0%;&#10;margin:20px 0 0 20px;" ); //Top left
	$css[1] = esc_html(	"top:0%;&#10;right:0%;&#10;margin:20px 20px 0 0;" ); //Top right
	$css[2] = esc_html(	"bottom:0%;&#10;left:0%;&#10;margin:0 0 20px 20px;" ); //Bottom left
	$css[3] = esc_html(	"bottom:0%;&#10;right:0%;&#10;margin:0 20px 20px 0;" ); //Bottom right
	$css[4] = esc_html(	"top:0;&#10;left:0;&#10;right:0;&#10;bottom:0;&#10;margin:auto;" ); //Middle
	return $css;
}